package com.gzlh.background.client;

import cn.hutool.json.JSONUtil;
import com.gzlh.background.properties.BackgroundPropertiesConfig;
import com.gzlh.bus.EventBus;
import com.gzlh.bus.EventDataManager;
import com.gzlh.config.SystemObject;
import com.gzlh.utils.DeviceCache;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@ChannelHandler.Sharable
public class BackgroundClientHandler extends SimpleChannelInboundHandler<String> {

    private BackgroundClientNetty client;

    @Resource
    private EventBus eventBus;


    public BackgroundClientHandler(BackgroundClientNetty client) {
        this.client = client;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String msg) throws Exception {
        System.out.println("Client received: " + msg);
//        if (!"back message".equalsIgnoreCase(msg)){
//            eventBus.startEvent(msg);
//        }
    }


    protected void sendHeartbeat(ChannelHandlerContext context) {
//        向后台发送设备信息json字符串
        DeviceCache.add("channelCode", SystemObject.applicationConfig.getChannelCode());
        String jsonStr = JSONUtil.toJsonStr(DeviceCache.getCacheMap());
        context.writeAndFlush(jsonStr);
//        System.out.println(" sent ping msg to " + context.channel().remoteAddress());
    }


    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        // IdleStateHandler 所产生的 IdleStateEvent 的处理逻辑.
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent e = (IdleStateEvent) evt;
            switch (e.state()) {
                case READER_IDLE:
                    handleReaderIdle(ctx);
                    break;
                case WRITER_IDLE:
                    handleWriterIdle(ctx);
                    break;
                case ALL_IDLE:
                    handleAllIdle(ctx);
                    break;
                default:
                    break;
            }
        }
    }

    protected void handleReaderIdle(ChannelHandlerContext ctx) {
//        System.err.println("---READER_IDLE---");
    }

    protected void handleWriterIdle(ChannelHandlerContext ctx) {
        //写空闲时发送心跳包
        sendHeartbeat(ctx);

//        System.err.println("---WRITER_IDLE---");
    }

    protected void handleAllIdle(ChannelHandlerContext ctx) {
        System.err.println("---ALL_IDLE---");
//        sendHeartbeat(ctx);
    }


    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.error("-------------后端服务器连接断开,{}-----------，进行重连", ctx.channel().remoteAddress());
//        System.err.println("---后端服务器连接断开，" + ctx.channel().remoteAddress() + " ---");
        client.connect();
    }

}
