package com.gzlh.bus;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.capture.factory.CaptureFactory;
import com.gzlh.device.capture.properties.CapturePropertiesConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.config.dto.EventDTO;
import com.gzlh.device.infrared.config.RedPropertiesConfig;
import com.gzlh.device.infrared.factory.RedFactory;
import com.gzlh.device.led.factory.LedFactory;
import com.gzlh.device.plc.handler.PLCHadnler;
import com.gzlh.device.weighbridge.config.WeighbridgePropertiesConfig;
import com.gzlh.device.weighbridge.factory.WeighbridgeFactory;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
public class EventThread implements Runnable{
    private String eventName;

    public EventThread(String eventName) {
        this.eventName=eventName;
    }

    /**
     * 处理事件
     *
     * @param eventName 服务.事件名 比如led.ready
     */
    public void handlerEvent(String eventName) {
        SerialSetting serialSetting=EventConfig.serialSetting;
        LedFactory ledFactory= SystemObject.ledFactory;
        RedFactory redFactory=SystemObject.redFactory;
        WeighbridgeFactory weighbridgeFactory=SystemObject.weighbridgeFactory;
        WeighbridgePropertiesConfig weighbridgePropertiesConfig=SystemObject.weighbridgePropertiesConfig;
        RedPropertiesConfig redPropertiesConfig=SystemObject.redPropertiesConfig;
        PLCHadnler plcHadnler= SpringUtil.getBean(PLCHadnler.class);
        CaptureFactory captureFactory=SystemObject.captureFactory;
        CapturePropertiesConfig capturePropertiesConfig=SystemObject.capturePropertiesConfig;
        log.info("event:{}", eventName);
        EventConfig.eventDTOList.stream().filter(eventDTO -> StrUtil.equals(eventName, eventDTO.getName()))
                .findFirst().ifPresent(eventDTO -> {
            List<String> actionList = eventDTO.getActionList().getAction();
            //依次执行动作
            actionList.forEach(action -> {
                String module = StrUtil.subBefore(action, ".", true);
                String command = StrUtil.subAfter(action, ".", true);
                if (StrUtil.equals(module, ModuleEnum.INFRARED_MODULE.getModuleEn())) {
                    //动作属于红外
                    redFactory.handler(redPropertiesConfig.getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.LED_MODULE.getModuleEn())) {
                    //动作属于led
                    ledFactory.handler(serialSetting.getLed().getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn())) {
                    //动作属于地磅
                    weighbridgeFactory.handler(weighbridgePropertiesConfig.getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.CAPTURE_MODULE.getModuleEn())) {
                    //动作属于抓拍单元--车牌识别
                    captureFactory.handler(capturePropertiesConfig.getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.PLC_MODULE.getModuleEn())) {
                    //动作属于plc
                    plcHadnler.handlerAction(command);
                }  else if (StrUtil.equals(module, ModuleEnum.CENTER_MODULE.getModuleEn())) {
                    //动作属于中心总线
                    handlerAction(command);
                }
                ThreadUtil.sleep(250);
            });
        });

    }

    /**
     * 调度中心命令处理
     *
     * @param action
     */
    private void handlerAction(String action) {
        if (StrUtil.contains(action, CenterActionCommand.SLEEP_COMMAND)) {
            //休眠
            handlerSleep(action);
        }
    }

    /**
     * 调度中心休眠
     *
     * @param action
     */
    private void handlerSleep(String action) {
        String time = StrUtil.subBefore(StrUtil.subAfter(action, "(", true), ")", true);
        if (NumberUtil.isInteger(time)) {
            ThreadUtil.sleep(Long.parseLong(time));
        }
    }

    @Override
    public void run(){
        handlerEvent(eventName);
    }
}
